'use strict'

var Rule = require('./Rule')

/**
 * Stylesheet abstraction, contains rules, injects stylesheet into dom.
 *
 * @param {Object} [rules] object with selectors and declarations
 * @param {Object} [generateClasses] generate selectors instead of using rules property as selector.
 * @param {Object} [attributes] stylesheet element attributes
 * @api public
 */
function Stylesheet(rules, generateClasses, attributes) {
    if (typeof generateClasses == 'object') {
        attributes = generateClasses
        generateClasses = false
    }
    this.element = null
    this.attached = false
    this.generateClasses = generateClasses || false
    this.rules = {}
    this.attributes = attributes || {}
    this.classes = generateClasses ? {} : null
    if (!this.attributes.type) this.attributes.type = 'text/css'
    if (!this.attributes.media) this.attributes.media = 'screen'
    if (!this.attributes.title) this.attributes.title = 'Generated by jss.'

    if (rules) this.addRules(rules)
}

module.exports = Stylesheet

/**
 * Insert stylesheet element to render tree.
 *
 * @api public
 * @return {Stylesheet}
 */
Stylesheet.prototype.attach = function () {
    if (this.attached) return this

    if (!this.element) {
        this.element = this.createElement()
        this.element.innerHTML = this.toString()
    }

    document.head.appendChild(this.element)
    this.attached = true

    return this
}

/**
 * Remove stylesheet element from render tree.
 *
 * @return {Stylesheet}
 * @api public
 */
Stylesheet.prototype.detach = function () {
    if (!this.attached) return this

    this.element.parentNode.removeChild(this.element)
    this.attached = false

    return this
}

/**
 * Add a rule to the current stylesheet.
 *
 * @param {Object} [selector] if you don't pass selector - it will be generated
 * @param {Object} style property/value hash
 * @return {Stylesheet}
 * @api public
 */
Stylesheet.prototype.addRule = function (selector, style) {
    var rule = new Rule(selector, style, this)
    var sheet = this.element.sheet
    sheet.insertRule(rule.toString(), sheet.cssRules.length)
    this.rules[rule.selector] = rule

    return rule
}

/**
 * Get a rule.
 *
 * @param {String} selector
 * @return {Rule}
 * @api public
 */
Stylesheet.prototype.getRule = function (selector) {
    return this.rules[selector]
}

/**
 * Add rules to the current stylesheet.
 *
 * @param {Object} rules selector:style hash.
 * @return {Stylesheet} this
 * @api public
 */
Stylesheet.prototype.addRules = function (rules) {
    var selector
    for (var key in rules) {
        if (!this.generateClasses) selector = key
        var rule = new Rule(selector, rules[key], this)
        this.rules[rule.selector] = rule
        rule.runPreprocessors()
        if (this.generateClasses) this.classes[key] = rule.className
    }

    return this
}

/**
 * Convert rules to a css string.
 *
 * @api public
 * @return {String}
 */
Stylesheet.prototype.toString = function () {
    var str = ''
    var rules = this.rules

    for (var selector in rules) {
        if (str) str += '\n'
        str += rules[selector].toString()
    }

    return str
}

/**
 * Create stylesheet element.
 *
 * @api private
 * @return {Element}
 */
Stylesheet.prototype.createElement = function () {
    var el = document.createElement('style')
    for (var name in this.attributes) el.setAttribute(name, this.attributes[name])

    return el
}
